import datetime
import requests
import urllib

class SpreeApi(object):
  def __init__(self, endpoint, api_key):
    self.endpoint = endpoint
    self.api_key = api_key

  def list_orders(self, since_id=0, _id=None, since_time=None, state='complete', page=1):
    if _id is not None:
      endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders?q[id_eq]=%s&q[state_eq]=%s' % (_id, state))
    elif since_time is not None:
      endpoint_url = self.append_api_key(
        self.endpoint + '/api/v1/orders?q[completed_at_gt]=%s&q[state_eq]=%s&q[s]=completed_at:asc' % (since_time, state))
    else:
      endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders?q[id_gt]=%s&q[state_eq]=%s' % (since_id, state))
    if page:
      endpoint_url = '{}&page={}'.format(endpoint_url, page)

    response = requests.get(endpoint_url)
    return response.json()

  def get_order(self, order_id):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s' % order_id)

    return requests.get(endpoint_url).json()

  def list_authorized_orders(self, days_back=14):
    start_date = datetime.datetime.utcnow() - datetime.timedelta(days=days_back)
    start_date = start_date.strftime("%Y-%m-%dT00:00:00")

    action = '/api/v1/orders?q[state_eq]=complete&q[payment_state_eq]=balance_due&q[completed_at_gt]=%s&per_page=1000' % start_date
    endpoint_url = self.append_api_key(self.endpoint + action)
    response = requests.get(endpoint_url)
    authorized_orders = response.json()
    if 'orders' not in authorized_orders:
      return None

    return [order for order in authorized_orders['orders'] if order['payments'][-1]['state'] == 'pending']

  def list_unshipped_orders(self, days_back=14):
    start_date = datetime.datetime.utcnow() - datetime.timedelta(days=days_back)
    start_date = start_date.strftime("%Y-%m-%dT00:00:00")
    action = '/api/v1/orders?q[state_eq]=complete&q[shipment_state_eq]=ready&q[completed_at_gt]=%s&per_page=1000' % start_date
    endpoint_url = self.append_api_key(self.endpoint + action)
    response = requests.get(endpoint_url)
    return response.json()

  def export_orders(self, since_id=None, since_time=None, page=1):
    if since_time is not None:
      endpoint_url = self.append_api_key(
        self.endpoint + '/api/v1/orders/export?since={}'.format(since_time))
    elif since_id is not None:
      endpoint_url = self.append_api_key(
        self.endpoint + '/api/v1/orders/export?since_id={}'.format(since_id))
    else:
      endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/export')

    if page:
      endpoint_url = '{}&page={}'.format(endpoint_url, page)

    response = requests.get(endpoint_url)

    return response.json()

  def mark_order_exported(self, payload):
    endpoint_url = self.append_api_key('{}/api/v1/orders/export'.format(self.endpoint))
    return requests.post(endpoint_url, json=payload)

  def import_sheet_orders(self, payload):
    endpoint_url = self.append_api_key('{}/api/v1/import_sheet_orders'.format(self.endpoint))
    return requests.post(endpoint_url, json={'orders': payload})

  def import_external_order(self, order_number, order_source, external_order_number, cost, carrier, tracking, account, code):
    payload = {
      'external_order_src': order_source,
      'external_order_id': external_order_number,
    }
    if tracking:
      payload['carrier'] = carrier
      payload['tracking'] = tracking
    if cost:
      payload['external_order_cost'] = cost

    if account:
      payload['external_order_account'] = account

    if code:
      payload['external_order_code'] = code

    endpoint_url = self.append_api_key('{}/api/v1/orders/{}/import_source'.format(self.endpoint, order_number))
    return requests.post(endpoint_url, json=payload)

  def add_order_tracking(self, order_number, carrier, tracking_number):
    action = '/api/v1/orders/{}/add_tracking?carrier={}&tracking={}'.format(order_number, carrier, tracking_number)
    endpoint_url = self.append_api_key(self.endpoint + action)
    response = requests.get(endpoint_url)
    return response.json()

  def list_payments(self, order_id):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s/payments' % order_id)
    response = requests.get(endpoint_url)
    return response.json()

  def get_authorized_payment_id(self, order_id):
    payments = self.list_payments(order_id)
    payments = [payment['number'] for payment in payments['payments'] if payment['state'] == 'pending']
    if len(payments) > 0:
      return payments[0]

    return None

  def capture_payment(self, order_id, payment_id):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s/payments/%s/capture' % (order_id, payment_id))
    response = requests.put(endpoint_url)
    return response.json()

  def void_payment(self, order_id):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s/payments/0/void' % order_id)
    response = requests.put(endpoint_url)
    return response.json()

  def cancel_order(self, order_id):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/orders/%s/cancel' % order_id)
    response = requests.put(endpoint_url)
    return response.json()

  def ship_order(self, shipment_id, tracking, carrier=''):
    carrier_upper = carrier.upper()
    if 'UPS' in carrier_upper:
        carrier = 'UPS'
    if 'USPS' in carrier_upper:
        carrier = 'USPS'
    if 'FEDEX' in carrier_upper:
        carrier = 'FedEx'
    if 'DHL' in carrier_upper:
        carrier = 'DHL'
    if 'AMAZON' in carrier_upper:
        carrier = 'Other'
    endpoint_url = self.append_api_key(
      self.endpoint + '/api/v1/shipments/%s/ship?shipment[tracking]=%s&shipment[carrier]=%s' % (shipment_id, tracking, carrier))
    response = requests.put(endpoint_url)
    return response.json()

  def import_product(self, product_data):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/import')

    return requests.post(endpoint_url, json=product_data)

  def create_product(self, product_data):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products')

    return requests.post(endpoint_url, json=product_data)

  def list_products(self, page=1, per_page=250):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products')
    payloads = {'page': page, 'per_page': per_page}

    return requests.get(endpoint_url, payloads).json()

  def list_inventories(self, page=1, per_page=250, since_id=None, **kwargs):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/inventories')
    payloads = {'per_page': per_page, 'q[s]': 'id:asc'}
    if since_id:
      payloads['q[id_gt]'] = since_id
    else:
      payloads['page'] = page

    payloads.update(kwargs)

    resp = requests.get(endpoint_url, payloads)
    try:
      result = resp.json()
    except:
      result = resp.text

    return result

  def list_stores(self):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/stores')
    return requests.get(endpoint_url).json()

  def get_product(self, product_id):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/%s' % product_id)

    return requests.get(endpoint_url).json()

  def get_product_by_sku(self, sku):
    endpoint_url = self.append_api_key(
      self.endpoint + '/api/v1/products?per_page=1&page=1&q[variants_including_master_sku_cont]=%s' % sku)
    products = requests.get(endpoint_url).json()
    if 'products' not in products or len(products['products']) == 0:
      return None

    return products['products'][0]

  def get_product_by_name(self, name):
    endpoint_url = self.append_api_key(
      self.endpoint + '/api/v1/products?per_page=1&page=1&q[search_by_name]=%s' % urllib.parse.quote(name))
    products = requests.get(endpoint_url).json()
    if 'products' not in products or len(products['products']) == 0:
      return None

    return products['products'][0]

  def delete_product(self, product_id):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/%s' % product_id)

    return requests.delete(endpoint_url)

  def update_product(self, product_id, product_data):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/%s' % product_id)

    return requests.put(endpoint_url, json=product_data)

  def create_properties(self, product_id, name, value):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/%s/product_properties/%s' % (product_id, name))
    product_data = {'product_property': {'value': value}}
    return requests.put(endpoint_url, json=product_data)

  def update_property(self, product_id, name, value):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/%s/product_properties/%s' % (product_id, name))
    product_data = {'product_property': {'value': value}}
    return requests.put(endpoint_url, json=product_data)

  def create_property(self, product_id, name, value):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/%s/product_properties' % product_id)
    product_data = {'product_property': {'value': value, 'property_name': name}}
    return requests.post(endpoint_url, json=product_data)

  def create_variant(self, product_id, sku, option_value_ids, price, cost):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/%s/variants' % product_id)
    data = {'variant': {'sku': sku, 'price': price, 'cost_price': cost, 'option_value_ids': [option_value_ids]}}
    return requests.post(endpoint_url, json=data)

  def update_variant(self, product_id, variant_id, **params):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/%s/variants/%s' % (product_id, variant_id))
    data = {'variant': params}
    return requests.put(endpoint_url, json=data)

  def update_variant_by_sku(self, sku, **params):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/variants/%s' % sku)
    data = {'variant': params}
    return requests.put(endpoint_url, json=data)

  def update_variant_sale_by_sku(self, sku, sale_rate):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/variants/%s' % sku)
    data = {'sale_rate': sale_rate}
    return requests.put(endpoint_url, json=data)

  def get_variant(self, product_id, sku, currency='USD'):
    endpoint_url = self.append_api_key(
        self.endpoint + '/api/v1/products/%s/variants?q[sku_eq]=%s&currency=%s' % (product_id, sku, currency))
    return requests.get(endpoint_url)

  def create_stock(self, variant_id, qty, stock_location=1, backorderable=False):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/stock_locations/%s/stock_items' % stock_location)
    data = {'stock_item': {'count_on_hand': qty, 'variant_id': variant_id, 'backorderable': backorderable}}
    return requests.post(endpoint_url, json=data)

  def update_stock(self, stock_item_id, qty, stock_location=1, force=True, backorderable=False):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/stock_locations/%s/stock_items/%s' % (stock_location, stock_item_id))
    data = {'stock_item': {'count_on_hand': qty, 'force': force, 'backorderable': backorderable}}
    return requests.put(endpoint_url, json=data)

  def create_taxonomy(self, name):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/taxonomies')
    data = {'taxonomy': {'name': name}}
    return requests.post(endpoint_url, json=data)

  def find_taxonomy_by_name(self, name):
    try:
      endpoint = '%s/api/v1/taxonomies?q[name_cont]=%s&page=1&per_page=1' % (self.endpoint, name)
      endpoint = self.append_api_key(endpoint)
      response = requests.get(endpoint, timeout=180).json()
      if 'taxonomies' not in response or not response['taxonomies']:
        return

      return response['taxonomies'][0]
    except:
      pass

  def create_taxon(self, taxonomy_id, name, parent_id=None):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/taxonomies/%s/taxons' % taxonomy_id)
    data = {'taxon': {'name': name, 'parent_id': parent_id}}
    return requests.post(endpoint_url, json=data)

  def list_taxons(self, taxonomy_id, per_page=1):
    taxons = []
    page = 1

    while True:
      try:
        url = self.endpoint + '/api/v1/taxonomies/%s/taxons?per_page=%s&page=%s' % (taxonomy_id, per_page, page)
        endpoint_url = self.append_api_key(url)
        response = requests.get(endpoint_url)
        json_data = response.json()
        taxons.extend(json_data['taxons'])
        if len(json_data['taxons']) < per_page:
          break
        page += 1
      except:
        continue

    taxons = self.loop_taxons(taxons, dict())

    return taxons

  def loop_taxons(self, taxons_response, taxons, parent_name=None):
    for taxon in taxons_response:
      key = taxon['name']
      if parent_name is not None:
        key = parent_name + '>' + key
      taxons[key] = taxon['id']
      if 'taxons' in taxon:
        taxons = self.loop_taxons(taxon['taxons'], taxons, key)

    return taxons

  def find_taxon_by_name(self, name, parent_id, taxonomy_id=1):
    try:
      endpoint = '%s/api/v1/taxonomies/%s/taxons?q[name_cont]=%s&without_children=1' % (self.endpoint, taxonomy_id, name)
      endpoint = self.append_api_key(endpoint)
      response = requests.get(endpoint, timeout=180).json()
      for taxon in response['taxons']:
        if taxon['parent_id'] == parent_id and taxon['name'] == name:
          return taxon

      for taxon in response['taxons']:
        if taxon['name'] == name:
          return taxon
    except:
      pass

  def add_review(self, product_id, review):
    review_data = {
      'product_id': product_id,
      'review': review
    }
    endpoint_url = self.append_api_key(
      self.endpoint + '/api/v1/reviews')
    return requests.post(endpoint_url, json=review_data)

  def add_review_by_sku(self, sku, review):
    review_data = {
      'sku': sku,
      'review': review
    }
    endpoint_url = self.append_api_key(
      self.endpoint + '/api/v1/reviews')

    return requests.post(endpoint_url, json=review_data)

  def get_product_reviews(self, product_id):
    endpoint_url = self.append_api_key(
      self.endpoint + '/api/v1/reviews?per_page=10&page=1&q[product_id_eq]=%s' % product_id)
    reviews = requests.get(endpoint_url).json()
    if 'reviews' not in reviews or len(reviews['reviews']) == 0:
      return None

    return reviews['reviews']

  def append_api_key(self, url):
    separator = '&' if '?' in url else '?'
    return url + '%stoken=%s' % (separator, self.api_key)
