import json
import datetime
import pdb

from google.oauth2 import service_account
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.oauth2 import _SERVICE_ACCOUNT_SCOPES

from em_tasks import logger
from em_tasks.store.models import *


class GoogleAdsInsightsTask():
  def __init__(self, customer_id, developer_token, version='v15', days=1):
    self.end_date = datetime.date.today() + datetime.timedelta(days=1)
    self.start_date = self.end_date - datetime.timedelta(days=days)

    ga_account = GoogleAdsAccount.get_or_none(GoogleAdsAccount.customer_id == customer_id)

    self.ga_id = ga_account.id
    self.impersonated_email = ga_account.impersonated_email
    self.customer_id = customer_id
    self.active = ga_account and ga_account.active

    service_account_info = json.loads(ga_account.service_account_data)
    credentials = service_account.Credentials.from_service_account_info(
      service_account_info,
      subject=self.impersonated_email,
      scopes=_SERVICE_ACCOUNT_SCOPES)
    self.ga_client = GoogleAdsClient(credentials, developer_token, version=version)
    self.ga_service = self.ga_client.get_service("GoogleAdsService")

  def run(self):
    if not self.active:
      return

    insights = []

    where_query = 'WHERE segments.date BETWEEN "{}" AND "{}"'.format(
        self.start_date.strftime('%Y-%m-%d'), self.end_date.strftime('%Y-%m-%d'))
    # where_query += ' AND metrics.clicks > 0'
    logger.debug('[WhereQuery] %s', where_query)

    query = """
      SELECT
        customer.id,
        metrics.impressions,
        metrics.clicks,
        metrics.ctr,
        metrics.conversions,
        metrics.cost_micros,
        metrics.average_cpc,
        metrics.average_cpm,
        segments.date
      FROM customer
      {}""".format(where_query)

    # Issues a search request using streaming.
    search_request = self.ga_client.get_type("SearchGoogleAdsStreamRequest")
    search_request.customer_id = self.customer_id
    search_request.query = query
    try:
      stream = self.ga_service.search_stream(search_request)
      for batch in stream:
        for row in batch.results:
          segments = row.segments
          metrics = row.metrics

          insight = {
            'impressions': metrics.impressions,
            'clicks': metrics.clicks,
            'conversions': metrics.conversions,
            'spend': metrics.cost_micros * 1.0 / 10**6,
            'ctr': metrics.ctr * 100,
            'cpc': metrics.average_cpc * 1.0 / 10**6,
            'cpm': metrics.average_cpm * 1.0 / 10**6,
          }

          s_date = segments.date
          ga_insight = GoogleAdsInsight.get_or_none((GoogleAdsInsight.google_ads_account_id == self.ga_id) & (GoogleAdsInsight.date == s_date))
          if not ga_insight:
            ga_insight = GoogleAdsInsight()
            ga_insight.google_ads_account_id = self.ga_id
            ga_insight.date = s_date

          for k, v in insight.items():
            if getattr(ga_insight, k) != v:
              insight_changed = True
              break

          if not insight_changed:
            continue

          for k, v in insight.items():
            setattr(ga_insight, k, v)
          ga_insight.save()
    except GoogleAdsException as ex:
      message = 'Request with ID "{}" failed with status "{}" and includes the following errors:'.format(
        ex.request_id, ex.error.code().name)

      errors = []
      for error in ex.failure.errors:
        error_msg = 'Error with message {}.'.format(error.message)
        if error.location:
          error_msg += "\n"
          for field_path_element in error.location.field_path_elements:
            error_msg += "On field: {}\n".format(field_path_element.field_name)

        errors.append(error_msg)

      logger.error(message + "\n" + "\n".join(errors))
